#ifndef HRP2_DYNAMIC_ROBOT_H
#define HRP2_DYNAMIC_ROBOT_H

template<class T>
class HRP2DynamicRobot : public T
{
public:
    /**
       \brief Get the upper bound for ith dof.
    */
    virtual double upperBoundDof(unsigned int inRankInConfiguration,
				 const vectorN& inConfig){
	double q, limit, ub;
	ub = T::upperBoundDof(inRankInConfiguration);
	if (inRankInConfiguration==(12+6)){ // CHEST_JOINT0
	    q = inConfig(inRankInConfiguration+1);
	    if (q > 0.76){
		limit = tan(-(q-0.91)/0.1)/18+0.2;
		if (limit < ub) ub = limit;
	    }
	}else if (inRankInConfiguration==(7+6)){ // LLEG_JOINT1
	    q = inConfig(inRankInConfiguration+1);
	    limit = 0.2182*atan((q+1.6406)*5.7296)+0.27925;
	    if (limit < ub) ub = limit;
	}else if (inRankInConfiguration==(13+6)){ // CHEST_JOINT1
	    q = inConfig(inRankInConfiguration-1);
	    if(q<0)
		limit =  0.1*atan((q+0.2)*18)+0.91;
	    else
		limit = -0.1*atan((q-0.2)*18)+0.91;
	    if (limit < ub) ub = limit;
	}else if (inRankInConfiguration==(11+6)){ // LLEG_JOINT5
	    q = inConfig(inRankInConfiguration-1);
	    limit = -0.0024/((q+1.4)*(q+1.4))+0.35;
	    if (limit < ub) ub = limit;
	}
	return ub;
    }
    /**
       \brief Get the lower bound for ith dof.
    */
    virtual double lowerBoundDof(unsigned int inRankInConfiguration,
				 const vectorN& inConfig){
	double q, limit, lb;
	lb = T::lowerBoundDof(inRankInConfiguration);
	if (inRankInConfiguration==(1+6)){ // RLEG_JOINT1
	    q = inConfig(inRankInConfiguration+1);
	    limit = -0.2182*atan((q+1.6406)*5.7296)-0.27925;
	    if (limit > lb) lb = limit;
	}else if (inRankInConfiguration==(2+6)){ // RLEG_JOINT2
	    q = inConfig(inRankInConfiguration-1);
	    if (q < 0.05){
		limit = tan(-(q+0.27925)/0.2182)/5.7296-1.6406;
		if (limit > lb) lb = limit;
	    }
	}else if (inRankInConfiguration==(8+6)){ // LLEG_JOINT2
	    q = inConfig(inRankInConfiguration-1);
	    if (q > 0.05){
		limit = tan((q-0.27925)/0.2182)/5.7296-1.6406;
		if (limit > lb) lb = limit;
	    }
	}else if (inRankInConfiguration==(4+6)){ // LLEG_JOINT4
	    q = inConfig(inRankInConfiguration+1);
	    limit = sqrt( 0.0024/(q+0.35))-1.4;
	    if (limit > lb) lb = limit;
	}else if (inRankInConfiguration==(5+6)){ // RLEG_JOINT5
	    q = inConfig(inRankInConfiguration-1);
	    limit =  0.0024/((q+1.4)*(q+1.4))-0.35;
	    if (limit > lb) lb = limit;
	}else if (inRankInConfiguration==(10+6)){ // LLEG_JOINT4
	    q = inConfig(inRankInConfiguration+1);
	    limit = sqrt(-0.0024/(q-0.35))-1.4;
	    if (limit > lb) lb = limit;
	}else if (inRankInConfiguration==(12+6)){ // CHEST_JOINT0
	    q = inConfig(inRankInConfiguration+1);
	    if (q > 0.76){
		limit = tan( (q-0.91)/0.1)/18-0.2;
		if (limit > lb) lb = limit;
	    }
	}
	return lb;
    }
};

#endif
