/*	$Id: gikDebug.cc 2007/11/19 18:41:55 mallet $ */

/* 
 * Copyright (c) 2007 LAAS/CNRS                       --  Mon Nov 19 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#ifdef GIK_DEBUG

#include <err.h>
#include <portLib.h>

#include "server/gikHeader.h"

#include "PaletteGik.h"


/*------------------------------------------------------------------------
 * Noise
 *
 * Description:
 *
 * Reports:      OK
 */

/* NoiseMain  -  codel EXEC of Noise
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
NoiseMain ( int *report )
{
    vectorN curConf = robot->currentConfiguration();
    vectorN confNoise ( curConf.size()-6 );
    double noiseAmp = 0.03;

    for ( unsigned int i=6; i<curConf.size();i++ )
    {
        confNoise ( i-6 ) = noiseAmp* ( ( ( double ) rand() ) / ( ( double ) RAND_MAX ) - 0.5 );
        curConf ( i ) = curConf ( i ) + confNoise ( i-6 );
    }

    paletteGik->setConfiguration ( curConf );

    return ETHER;
}


/*------------------------------------------------------------------------
 * Config
 *
 * Description:
 *
 * Reports:      OK
 */

/* ConfigMain  -  codel EXEC of Config
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ConfigMain ( int *report )
{
    vectorN curConf ( 46 );

    curConf ( 0 ) = 0.0;
    curConf ( 1 ) = 0.0;
    curConf ( 2 ) = 0.0;

    curConf ( 3 ) = -0.0188144646511;
    curConf ( 4 ) = 0.0120040616277;
    curConf ( 5 ) = -1.52451139904;

    curConf ( 6 ) = 0.00269980618668;
    curConf ( 7 ) = -0.000157079632679;
    curConf ( 8 ) = -0.409000456912;
    curConf ( 9 ) = 0.869184439469;
    curConf ( 10 ) = -0.417552570247;
    curConf ( 11 ) = 0.00106814150222;
    curConf ( 12 ) = -0.00286343080405;
    curConf ( 13 ) = -0.0167289808804;
    curConf ( 14 ) = -0.44638540949;
    curConf ( 15 ) = 0.871540633959;
    curConf ( 16 ) = -0.417482757077;
    curConf ( 17 ) = 0.00106814150222;
    curConf ( 18 ) = 0.000877320137069;
    curConf ( 19 ) = 0.00449577653827;
    curConf ( 20 ) = 0.000981747704247;
    curConf ( 21 ) = 0.00920388472731;
    curConf ( 22 ) = 0.26295773933;
    curConf ( 23 ) = -0.171354774974;
    curConf ( 24 ) = 0.0120166670041;
    curConf ( 25 ) = -0.52427781776;
    curConf ( 26 ) = 0.00156083021495;
    curConf ( 27 ) = 0.00601390593687;
    curConf ( 28 ) = 0.170302158775;
    curConf ( 29 ) = 0.259977073947;
    curConf ( 30 ) = 0.175653237895;
    curConf ( 31 ) = -0.0143728762206;
    curConf ( 32 ) = -0.524400536223;
    curConf ( 33 ) = 0.00185612241778;
    curConf ( 34 ) = -0.00637294509728;
    curConf ( 35 ) = 0.170401576264;
    curConf ( 36 ) = -0.170302158775;
    curConf ( 37 ) = 0.170302158775;
    curConf ( 38 ) = -0.170302158775;
    curConf ( 39 ) = 0.170302158775;
    curConf ( 40 ) = -0.170302158775;
    curConf ( 41 ) = -0.170401576264;
    curConf ( 42 ) = 0.170401576264;
    curConf ( 43 ) = -0.170401576264;
    curConf ( 44 ) = 0.170401576264;
    curConf ( 45 ) = -0.170401576264;

    paletteGik->setConfiguration ( curConf );

    return ETHER;
}


/*------------------------------------------------------------------------
 * Reset
 *
 * Description:
 *
 * Reports:      OK
 */

/* ResetMain  -  codel EXEC of Reset
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ResetMain ( int *report )
{
    paletteGik->reset();
    return ETHER;
}


/*------------------------------------------------------------------------
 * Stepback
 *
 * Description:
 *
 * Reports:      OK
 */

/* StepbackMain  -  codel EXEC of Stepback
        Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
StepbackMain ( BOOLEAN *outSuccess, int *report )
{
    bool solved = paletteGik->stepback();

    if ( solved )
        *outSuccess = True;
    else
        *outSuccess = False;
    return ETHER;
}


/*------------------------------------------------------------------------
 * Snoop
 *
 * Description:
 *
 * Reports:      OK
 */

/* SnoopMain  -  codel EXEC of Snoop
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SnoopMain ( SNOOPDATA *snoop_input, BOOLEAN *outSuccess, int *report )
{
    vector3d p;

    worldCoordinates ( snoop_input->gazepoint,p );

    bool solved = paletteGik->snoop ( snoop_input->headheight, snoop_input->headadvancement, p );

    if ( solved )
        *outSuccess = True;
    else
        *outSuccess = False;
    return ETHER;
}

#endif /* GIK_DEBUG */
