/*	$Id: gikStruct.h 2008/04/16 14:38:47 mallet $ */

/*
 * Copyright (c) 2007 Oussama Kanoun                  --  Mon Apr 16 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef GIK_STRUCT_H
#define GIK_STRUCT_H

typedef struct GikString {
  char name[1024];
} GikString;


/* System state */
typedef enum BOOLEAN
{
    False = 0,
    True  = 1
}BOOLEAN;

/** Left or right hand */
typedef enum GikHand {
  GIK_LEFT_HAND,
  GIK_RIGHT_HAND
} GikHand;

typedef struct VEC
{
    double x;
    double y;
    double z;
}
VEC;

/** Reach request parameters */
typedef struct GikReachData {
  GikHand hand;			/**< the hand to place */
  VEC position;			/**< 3D position of the hand */

  BOOLEAN enableStepping;	/**< Allow one optional step if true */

  BOOLEAN orient_job;		/**< Constraint hand orientation if true */
  VEC axis;			/**< Hand orientation axis */

  BOOLEAN look_job;		/**< Constraint gaze point if true */
  VEC gazepoint;		/**< Gaze point */
} GikReachData;

/** Grasp request parameters */
typedef struct GikGraspData
{
  GikHand hand;			/**< the hand to use */
  VEC position;			/**< 3D position of the hand */
  double clench;		/**< Hand clench value */
} GikGraspData;

/** GraspVia request parameters */
typedef struct GikGraspViaData
{
  GikHand hand;			/**< the hand to use */
  VEC via;			/**< via 3D position of the hand */
  VEC position;			/**< 3D position of the hand */
  double clench;		/**< Hand clench value */
} GikGraspViaData;

typedef struct CLENCHDATA
{
    BOOLEAN right;
    double  clench;
}
CLENCHDATA;

/** TrackSpeed request parameters */
typedef struct GikTrackSpeed {
  GikHand hand;
  GikString objectName;
  GikString wristName;
  double speed;
  double delay;
} GikTrackSpeed;

/** TrackHands request parameters */
typedef struct GikTrackHands {
  GikString file;
  GikString dump;
  double period;
} GikTrackHands;

#ifdef GIK_DEBUG
typedef struct SNOOPDATA {
  double  headheight;
  double  headadvancement;
  VEC     gazepoint;
} SNOOPDATA;
#endif /* GIK_DEBUG */


/* --- Magic stuff --------------------------------------------------- */

#define GIK_TARGET_CONFIG_NAME		"gik-target"

#endif /* GIK_STRUCT_H */
